@@ -55,7 +55,6 @@ done:
 }
 
 struct m_pedit_util p_pedit_icmp = {
-	NULL,
-	"icmp",
-	parse_icmp,
+	.id = "icmp",
+	.parse_peopt = parse_icmp,
 };
